{% extends "main.html" %}

{% block box %}

<h2>{% i18nc "%1 is version" "Welcome to KMail %1" _(version) %}</h2>

<p>{% i18n "KMail is the email client by KDE. It is designed to be fully compatible with Internet mailing standards including IMAP, POP3 and SMTP." %}
</p>

<p>{% i18n_var "KMail has many powerful features that are described in the <a href='help:/kmail2/index.html'>documentation</a>." as features %}
{{ features|safe }}
</p>


{% if importantChanges|length %}
<p>{% i18n "Important changes since last version:" %}</p>
<ul>
    {% for change in importantChanges %}
    <li>{{ change|safe }}</li>
    {% endfor %}
</ul>
{% endif %}

{% if newFeatures|length %}
<p>{% i18n "Some of the new features in this release of KMail include:" %}</p>
<ul>
    {% for feature in newFeatures %}
    <li>{{ feature|safe }}</li>
    {% endfor %}
</ul>
{% endif %}

{% if firstStart %}
<p>{% i18n "Please take a moment to fill in the KMail configuration panel at Settings->Configure KMail. You need to create at least a default identity and incoming as well as outgoing mail account." %}
</p>
{% endif %}

<p>{% i18n "We hope that you will enjoy KMail." %}</p>
<p>{% i18n "Thank you," %}</p>
<p>{% i18n "The KMail Team" %}</p>


{% endblock box %}
