/*
 *   kmail: KDE mail client
 *   Copyright (C) 2000 Espen Sand, espen@kde.org
 *   Copyright (C) 2001-2003 Marc Mutz, mutz@kde.org
 *   Contains code segments and ideas from earlier kmail dialog code.
 *   Copyright (C) 2010 Volker Krause <vkrause@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#ifndef NEWIDENTITYDIALOG_H
#define NEWIDENTITYDIALOG_H

#include <QDialog>

class QComboBox;
class QLineEdit;
class QButtonGroup;

namespace KIdentityManagement {
class IdentityManager;
}

namespace KMail {
class NewIdentityDialog : public QDialog
{
    Q_OBJECT

public:

    enum DuplicateMode {
        Empty = 0,
        ControlCenter,
        ExistingEntry
    };
    explicit NewIdentityDialog(KIdentityManagement::IdentityManager *manager, QWidget *parent = nullptr);

    Q_REQUIRED_RESULT QString identityName() const;
    Q_REQUIRED_RESULT QString duplicateIdentity() const;
    Q_REQUIRED_RESULT DuplicateMode duplicateMode() const;

private:
    void slotHelp();
    void slotEnableOK(const QString &);
    QLineEdit *mLineEdit = nullptr;
    QComboBox *mComboBox = nullptr;
    QButtonGroup *mButtonGroup = nullptr;
    KIdentityManagement::IdentityManager *mIdentityManager = nullptr;
    QPushButton *mOkButton = nullptr;
};
}

#endif
