<!-- This file should be synchronized with kmail_part.rc to provide
     the same menu entries at the same place in KMail and Kontact  -->

<!DOCTYPE gui>
<gui version="546" name="kmmainwin" translationDomain="kmail">
 <MenuBar>
  <Menu noMerge="1" name="file" >
   <text>&amp;File</text>
   <Menu name="file_new" >
     <text context="@title:menu New message, folder or new window.">New</text>
     <Action name="new_message" />
     <Action name="new_from_template" />
     <Separator/>
     <Action name="akonadi_collection_create" />
     <Separator/>
     <Action name="kmail_new_addressbook_contact" />
   </Menu>
   <Separator/>
   <Action name="file_open" />
   <Action name="file_open_recent" />
   <Action name="file_save_as" />
   <Separator/>
   <Action name="file_export_pdf" />
   <Separator/>
   <Action name="import"/>
   <Separator/>
   <Action name="file_print_preview"/>
   <Action name="file_print" />
   <Separator/>
   <Action name="expire_all_folders" />
   <Action name="akonadi_empty_all_trash"/>
   <Separator/>
   <Action name="akonadi_work_offline" />
   <Separator/>
   <Action name="check_mail" />
   <Action name="check_mail_in" />
   <Action name="send_queued" />
   <Action name="send_queued_via" />
   <Separator/>
   <Action name="message_delayed"/>
   <Action name="followup_reminder_messages"/>
   <Separator/>
   <ActionList name="kmail_plugins_file"/>
   <Separator/>
   <Action name="file_quit" />
  </Menu>
  <Menu noMerge="1" name="edit" >
   <text>&amp;Edit</text>
   <Action name="kmail_undo" />
   <Action name="kmail_redo" />
   <Separator/>
   <Action name="kmail_copy" />
   <Separator/>
   <Action name="akonadi_collection_copy"/>
   <Action name="akonadi_collection_cut"/>
   <Separator/>
   <Action name="akonadi_item_copy"/>
   <Action name="akonadi_item_cut"/>
   <Separator/>
   <Action name="akonadi_paste"/>
   <Separator/>
   <Action name="akonadi_move_to_trash" />
   <Action name="move_thread_to_trash" />
   <Separator/>
   <Action name="find_in_messages" />
   <Separator/>
   <Action name="mark_all_messages" />
   <Action name="mark_all_text" />
   <Separator/>
   <ActionList name="kmail_plugins_edit"/>
  </Menu>
  <Menu noMerge="1" name="view">
   <text>&amp;View</text>
   <Action name="view_message_list"/>
   <Action name="view_headers"/>
   <Action name="view_attachments"/>
   <Separator/>
   <Action name="expand_thread"/>
   <Action name="collapse_thread"/>
   <Action name="expand_all_threads"/>
   <Action name="collapse_all_threads"/>
   <Separator/>
   <Action name="save_message_display_format"/>
   <Action name="reset_message_display_format"/>
   <Separator/>
   <Action name="disable_emoticon"/> 
   <Separator/>
   <Action name="toggle_fixedfont"/>
   <Separator/>
   <Action name="encoding" />
   <Separator/>
   <Action name="zoom_menu"/>
  </Menu>
  <Menu noMerge="1" name="go">
   <text>&amp;Go</text>
   <Action name="jump_to_folder"/>
   <Separator/>
   <Action name="go_next_message"/>
   <Action name="go_next_unread_message"/>
   <Action name="go_prev_message"/>
   <Action name="go_prev_unread_message"/>
   <Separator/>
   <Action name="go_next_unread_folder"/>
   <Action name="go_prev_unread_folder"/>
   <Separator/>
   <Action name="go_next_unread_text"/>
  </Menu>
  <Menu noMerge="1" name="folder" >
   <text>F&amp;older</text>
   <Action name="akonadi_collection_properties"/>
   <Separator/>
   <Action name="akonadi_collection_create" />
   <Separator/>
   <Action name="akonadi_mark_all_as_read" />
   <Action name="markallmessagereadcurentfolderandsubfolder" />
   <Separator/>
   <Action name="akonadi_collection_sync"/>
   <Action name="akonadi_collection_sync_recursive" />
   <Separator/>
   <Action name="akonadi_move_all_to_trash" />
   <Action name="akonadi_collection_delete" />
   <Separator/>
   <Action name="archive_folder" />
   <Separator/>
   <Action name="akonadi_remove_duplicates" />
   <Action name="remove_duplicate_recursive" />
   <Separator/>
   <Menu name="apply_filters_folder_actions">
     <text>Apply Filters on Folder</text>
     <Action name="apply_filters_folder" />
     <ActionList name="menu_filter_folder_actions" />
   </Menu>
   <Menu name="apply_filters_folder_recursive_actions">
     <text>Apply Filters on Folder and all its Subfolders</text>
     <Action name="apply_filters_folder_recursive" />
     <ActionList name="menu_filter_folder_recursive_actions" />
   </Menu>
   <Separator/>
   <Action name="display_format_message" />
   <Action name="prefer_html_external_refs" />
   <Action name="thread_messages" />
   <Action name="thread_messages_by_subject" />
   <Separator/>
   <Action name="folder_mailinglist_properties" />
   <Action name="folder_shortcut_command" />
   <Separator/>
   <ActionList name="kmail_plugins_folder"/>
  </Menu>
  <Menu noMerge="1" name="message" >
   <text>&amp;Message</text>
   <Action name="new_message" />
   <Action name="post_message" />
   <Separator/>
   <Action name="reply" />
   <Action name="reply_all" />
   <Menu noMerge="1" name="reply_special" >
     <text>Reply Special</text>
     <Action name="reply_author" />
     <Action name="reply_list" />
     <Action name="noquotereply" />
     <Separator/>
     <Action name="custom_reply" />
     <Action name="custom_reply_all" />
   </Menu>
   <Menu name="menubar_message_forward">
     <text>&amp;Forward</text>
     <ActionList name="forward_action_list"/>
     <Separator/>
     <Action name="custom_forward" />
   </Menu>
   <Action name="send_again" />
   <Action name="mailing_list"/>
   <Separator/>
   <Action name="akonadi_item_copy_to_menu" />
   <Action name="akonadi_item_move_to_menu" />
   <Action name="copy_decrypted_to_menu" />
   <Separator/>
   <Action name="set_status" />
   <Action name="thread_status" />
   <Separator/>
   <Action name="create_filter"/>
   <Menu name="apply_filter_actions" >
     <text>A&amp;pply Filter</text>
     <Action name="apply_filters" />
     <ActionList name="menu_filter_actions" />
   </Menu>
   <Separator/>
   <Action name="create_todo"/>
   <Action name="create_event"/>
   <Action name="create_note"/>
   <Separator/>
   <Action name="view_source"/>
   <Separator/>
   <ActionList name="kmail_plugins_message"/>
  </Menu>
  <Menu noMerge="1" name="tools">
   <text>&amp;Tools</text>
   <Action name="search_messages" />
   <Separator/>
   <Action name="addressbook"/>
   <Action name="tools_start_certman"/>
   <Separator/>
   <Action name="tools_edit_vacation"/>
   <Separator/>
   <Action name="tools_debug_sieve"/>
   <Action name="filter_log_viewer"/>
   <Separator/>
   <Action name="importWizard"/>
   <Separator/>
   <Action name="kmail_export_data"/>
   <Separator/>
   <ActionList name="kmail_plugins_tools"/>
  </Menu>
  <Menu noMerge="1" name="settings">
   <text>&amp;Settings</text>
   <Merge name="StandardToolBarMenuHandler"/>
   <Action name="options_show_menubar" group="settings_configure" />
   <Action name="show_quick_search"/>
   <Action name="filter" append="save_merge"/>
   <Action name="sieveFilters" append="save_merge"/>
   <Action name="accountWizard"/>
   <Separator/>
   <Action name="tools_automatic_archiving"/>
   <Separator/>
   <Action name="switch_application_language" group="settings_configure"/>
   <Action name="options_configure_keybinding" group="settings_configure"/>
   <Action name="kmail_configure_notifications" group="settings_configure"/>
   <Action name="options_configure_toolbars" group="settings_configure" />
   <Action name="kmail_configure_kmail" group="settings_configure"/>
  </Menu>
  <Menu name="help">
   <text>&amp;Help</text>
   <Action name="help_kmail_welcomepage"/>
  </Menu>
 </MenuBar>
 <Menu name="akonadi_favoriteview_contextmenu">
  <Action name="akonadi_collection_sync"/>
  <Separator/>
  <Action name="akonadi_mark_all_as_read" />
  <Action name="search_messages" />
  <Action name="akonadi_move_to_trash" />
  <Separator/>
  <Action name="akonadi_collection_remove_from_favorites"/> 
  <Action name="akonadi_collection_rename_favorite"/>
  <Separator/>
  <Action name="expire_settings"/>
  <Action name="folder_shortcut_command"/>
  <Action name="akonadi_collection_properties"/>
  <Separator/>
  <Action name="favorite_mode"/>
 </Menu>

 <Menu name="akonadi_favoriteview_emptyselection_contextmenu">
  <Action name="add_favorite_folder" />
  <Separator/>
  <Action name="akonadi_collection_sync_favorite_folders" />
  <Separator/>
  <Action name="favorite_icon_size"/>
  <Separator/>
  <Action name="favorite_mode"/>
 </Menu>

 <Menu name="akonadi_collectionview_contextmenu">
  <ActionList name="akonadi_collection_sync_actionlist"/>
  <Separator/>
  <Action name="akonadi_mark_all_as_read" />
  <Action name="search_messages" />
  <ActionList name="outbox_folder_actionlist" />
  <Action name="akonadi_move_all_to_trash"/>
  <Separator/>
  <ActionList name="akonadi_collection_move_copy_menu_actionlist"/>
  <Action name="akonadi_collection_delete"/>
  <Separator/>
  <Menu name="apply_filters_folder_actions">
   <text>Apply Filters on Folder</text>
   <Action name="apply_filters_folder" />
   <ActionList name="menu_filter_folder_actions" />
  </Menu>
  <Menu name="apply_filters_folder_recursive_actions">
   <text>Apply Filters on Folder and all its Subfolders</text>
   <Action name="apply_filters_folder_recursive" />
   <ActionList name="menu_filter_folder_recursive_actions" />
  </Menu>
  <Separator/>
  <ActionList name="akonadi_collection_add_to_favorites_actionlist"/>
  <Separator/>
  <Action name="expire_settings"/>
  <Action name="archive_folder" />
  <Action name="folder_shortcut_command"/>
  <ActionList name="akonadi_collection_collectionproperties_actionlist"/>
  <ActionList name="resource_settings" />
  <ActionList name="resource_restart" />
  <Separator/>
  <ActionList name="collectionview_actionlist" />
 </Menu>

 <Menu name="akonadi_messagelist_contextmenu">
  <Action name="message_reply_menu"/>
  <Action name="message_forward"/>
  <ActionList name="messagelist_actionlist" />
  <Separator/>
  <action name="set_status"/>
  <action name="thread_status"/>
  <Separator/>
  <action name="create_filter"/>
  <action name="apply_filter_actions"/>
  <Separator/>
  <action name="file_save_as"/>
  <Separator/>
  <Action name="file_print_preview"/>
  <Action name="file_print"/>
  <Separator/>
  <Action name="akonadi_move_to_trash"/>
  <Action name="move_thread_to_trash"/>
  <Separator/>
  <Action name="akonadi_item_move_to_menu"/>
  <Action name="akonadi_item_copy_to_menu"/>
  <Action name="copy_decrypted_to_menu"/>
  <Separator/>
  <Action name="create_todo"/>
  <Action name="create_event"/>
  <Action name="create_note"/>
  <Separator/>
  <Action name="annotate"/>
  <Separator/>
  <Action name="message_followup_reminder"/>
 </Menu>

 <ToolBar noMerge="1" name="mainToolBar" fullWidth="true" >
  <text>Main Toolbar</text>
  <Action name="new_message" />
  <Action name="file_print" />
  <Separator/>
  <Action name="check_mail_in" />
  <Separator/>
  <Action name="message_reply_menu" />
  <Action name="message_forward"/>
  <Separator/>
  <Action name="akonadi_move_to_trash" />
  <Separator/>
  <ActionList name="toolbar_filter_actions" />
  <Separator/>
  <ActionList name="toolbar_messagetag_actions" />
  <Action name="create_todo"/>
  <Separator/>
  <ActionList name="kmail_toolbar_actions"/> 
 </ToolBar>
</gui>
